#!/bin/bash
#******************************************************************************#
# @file          : dataTunnelKey.sh
# @brief         : This file contains source code for Wireguard Key genaration
#                  for data tunnel.
#******************************************************************************#

. /lib/functions.sh "TUNNEL"

#Fetch the Caller Function name and Line number
logInfo() { echo "("${FUNCNAME[1]}":"${BASH_LINENO}")"; }

#******************************************************************************#
#module       : curlExec
#arg1         : API need to be called
#arg2         : Payload for API
#Description  : Used to call the API
#Return       : API response
#******************************************************************************#
curlExec()
{
    local apiName=$1
    local datarawParam=$2
    local apiResp=""
    local curlResp=""

    if [[ "$apiName" == "api-user/logout/" ]];then
        apiResp=$(curl --cacert $certFile --location \
            --connect-timeout 60 -m 60 --request POST \
            $mgmtProto://$mgmtUrl:$mgmtPort/$apiName \
            --header 'Authorization: '"$Key"'' -F refresh=$refreshKey)
        curlResp="$?"
        longDebug "$(logInfo)" "Curl Exit code for Logout API - $curlResp"
        longDebug "$(logInfo)" "logout Response: $apiResp"
    else
        apiResp=$(curl --cacert $certFile --location \
            --connect-timeout 60 -m 60 --request POST \
            $mgmtProto://$mgmtUrl:$mgmtPort/api-device/device/$apiName \
            --header  'Authorization: '"$Key"'' --header \
            'Content-Type: application/json' --data-raw $datarawParam)
        curlResp="$?"
        longDebug "$(logInfo)" "Curl Exit code for $apiName API - $curlResp"
        longDebug "$(logInfo)" "$apiName Response: $apiResp"
    fi
    echo $apiResp
}

#******************************************************************************#
# @function      : logoutAPI
# @brief         : This function is used API Authorization logout.
#******************************************************************************#
logoutAPI() {
   local retryCount=0

   refreshKey=`jq -r .refresh $keyFile`
   if [ -z "$refreshKey" -o "$refreshKey" = "null" ]
   then
       longErr "$(logInfo)" "Logout API refreshkey is $refreshKey"
       exit
   fi
   while :
   do
       if [ "$retryCount" -le 3 ]; then
           apiResp=$(curlExec "api-user/logout/")
           [[ -z "$apiResp" ]] || break
           sleep 5
           retryCount=`expr $retryCount + 1`
           [[ $retryCount -le 3 ]] && \
               longCrit "$(logInfo)" "Logout API response failed:" \
                   "Retrying... count-$retryCount"
       else
           longCrit "$(logInfo)" "Logout API response failed... Exiting..."
           rm -f $keyFile > /dev/null 2>&1
           exit
       fi
   done
   rm -f $keyFile > /dev/null 2>&1
}

#when script called from exe_ztp.sh, Get the Authorization key from CLA
Key=$1
#Get the total count of key's to be generated from EM
keyCount=$2
keyFile="/opt/inseego/etc/key"
wgKeyFile="/opt/inseego/etc/dTWireguardKey"
certFile="/etc/ssl/cert.pem"
deviceMac=`cat /sys/class/net/eth0/address` #For CBR
retryCount=0
logoutAPI_flag=0

if [ $keyCount -gt 0 ]
then
    #Multiple Wireguard key pair will be generated based on KeyCount(KeyCount > 0)
    longDebug "$(logInfo)" "Key's to be generated is $keyCount" \
        "Sending Key to EM via Ansible Response"
    pubKeyList=`echo "["`
    for (( ikey=0; ikey<$keyCount; ikey++ ))
    do
        privateKey=`wg genkey`
        encPrivateKey=`echo $privateKey | base64`
        publicKey=`echo $privateKey | wg pubkey`
        echo "$encPrivateKey $publicKey" >> $wgKeyFile
        pubKeyList=`echo "$pubKeyList'$publicKey',"`
    done
    pubKeyList=`echo $pubKeyList | sed '$s/,$//'`
    pubKeyList=`echo "$pubKeyList]"`
    echo "$pubKeyList"
    longDebug "$(logInfo)" "Generated_Key(s):$pubKeyList"
    exit
fi

#Fetch the EM details from ztp key
mgmtProto=`uci get ztp.EM_details.proto`
if [ -z $mgmtProto ]; then
    longErr "$(logInfo)" "Edge Manager IP not Found..."
    exit
fi
mgmtUrl=`uci get ztp.EM_details.domain`
if [ -z $mgmtUrl ]; then
    longErr "$(logInfo)" "Edge Manager URL not Found..."
    exit
fi
mgmtPort=`uci get ztp.EM_details.port`
if [ -z $mgmtPort ]; then
    longErr "$(logInfo)" "Edge Manager Port not Found..."
    exit
fi

longDebug "$(logInfo)" "Key's to be generated is $keyCount Sending Key to EM via API"

#when script called from playbook, Generate the Authorization key
if [[ $Key == "none" ]]; then
    longInfo "$(logInfo)" "Generating the Tokens..!!"
    /opt/inseego/bin/accessToken.sh
    accessKey=`jq -r .access $keyFile`
    if [ -z "$accessKey" -o "$accessKey" = "null" ]
    then
        longErr "$(logInfo)" "Login API response failed: AccessKey - $accessKey"
        exit
    fi
    Key="Bearer ${accessKey}"
    longDebug "$(logInfo)" "AccessKey - $accessKey"
    logoutAPI_flag=1
fi

#Single Wireguard key pair will be generated based on KeyCount(KeyCount == 1)
longDebug "$(logInfo)" "Generating Wireguard Public Key for EM....."
privateKey=`wg genkey`
encPrivateKey=`echo $privateKey | base64`
publicKey=`echo $privateKey | wg pubkey`
echo "$encPrivateKey $publicKey" >> $wgKeyFile

longDebug "$(logInfo)" "Wireguard key is generated" \
              "device_mac:$deviceMac,device_public_key:$publicKey"
#Send the Wireguard key Information to EM
while :
do
    if [ "$retryCount" -le 3 ]; then
        apiResp=$(curlExec "deviceWireguardInfo" '{"device_mac":"'$deviceMac'","device_public_key":"'$publicKey'"}')
        [[ -z "$apiResp" ]] || break
        sleep 5
        retryCount=`expr $retryCount + 1`
        [[ $retryCount -le 3 ]] && \
            longCrit "$(logInfo)" "WireguardInfo API response failed:" \
                "Retrying... count-$retryCount"
    else
        longCrit "$(logInfo)" "WireguardInfo API response failed..." \
            "Triggering Logout API and Exiting..."
        grep -v  "$publicKey" $wgKeyFile > tmpfile && mv tmpfile  $wgKeyFile
        break
    fi
done

resp=`echo $apiResp | jq -r .success`
if [[ "$resp" == "false" ]]; then
    longErr "$(logInfo)" "Wireguard Key exchange response Failed Response-$resp"
fi

#Call the logoutAPI only when the Access key is generated from this script
if [ "$logoutAPI_flag" == 1 ]; then
    longNotice "$(logInfo)" "Logout API is triggered"
    logoutAPI
fi
