#!/bin/bash

. /lib/functions.sh "ADMIN"

#Fetch the Caller Function name and Line number
logInfo() { echo "("${FUNCNAME[1]}":"${BASH_LINENO}")"; }

#******************************************************************************#
#module       : curlExec
#arg1         : API need to be called
#arg2         : Payload for API
#Description  : Used to call the API
#Return       : API response
#******************************************************************************#
curlExec()
{
    local apiName=$1
    local datarawParam=$2
    local apiResp=""
    local curlResp=""

    apiResp=$(curl --cacert $certFile --location \
        --connect-timeout 60 -m 60 --request POST \
        $mgmtProto://$mgmtUrl:$mgmtPort/api-device/device/$apiName \
        --header  'Authorization: '"$Key"'' --header \
        'Content-Type: application/json' --data-raw $datarawParam)
    curlResp="$?"
    longDebug "$(logInfo)" "Curl Exit code for $apiName API - $curlResp"
    longDebug "$(logInfo)" "$apiName Response: $apiResp"
    echo $apiResp
}

#******************************************************************************#
#module       : Firmware upgarde API
#arg1         : Operation ID of process received from EM
#arg2         : Severity of message send to EM
#arg3         : Remark message send to EM(must be single or with underscore)
#Description  : Used to send execution status of Firmware upgrade to EM
#******************************************************************************#
fw_api()
{
    operid=$1
    severity=$2
    remarks=$3
    local retryCount=0

    while :
    do
        if [ "$retryCount" -le 3 ]; then
            apiResp=$(curlExec "firmwareUpgradeStatus" \
                '{"operation_id":"'$operid'","severity":"'$severity'","remarks":"'$remarks'"}')
            [[ -z "$apiResp" ]] || break
            sleep 5
            retryCount=`expr $retryCount + 1`
            [[ $retryCount -le 3 ]] && \
                longCrit "$(logInfo)" "Firmware upgrade response empty: "\
                    "Retrying... count-$retryCount"
        else
            longCrit "$(logInfo)" "Firmware upgrade Response Failed... Exiting..."
            exit
        fi
    done
    longInfo "$(logInfo)" "${operid} Message Sent Successful"
}

#******************************************************************************#
#module       : Upgrading FOTA
#arg1         : md5sum of tar file
#arg2         : Operation ID of process received from EM
#Description  : To upgrade the image
#******************************************************************************#
upgrade()
{
    if [ "$#" -ne 2 ]; then
        longErr "$(logInfo)" "Insufficient Argument from playbook... Exiting..."
        fw_api "${operid}" "Error" "Insufficient_Argument_from_playbook"
        exit
    fi
    md5sum=$1
    operid=$2
    longNotice "$(logInfo)" "Firmware Upgrade Execution Started" \
        "md5sum of tar file=$md5sum operation id=$operid"

    #Download Firmware File
    rm /tmp/firmware.tar.gz > /dev/null 2>&1
    apiResp=$(curl --cacert $certFile --connect-timeout 60 -m 300 --retry 3 \
        --retry-delay 5 --header 'Authorization: '"$Key"'' --output /tmp/firmware.tar.gz \
        $mgmtProto://$mgmtUrl:$mgmtPort/api-device/device/firmwareDownload?operation_id=$operid)
    curlResp=$?
    longInfo "$(logInfo)" "Curl Exit code for Firmware download - "$curlResp""
    if [ "$curlResp" -eq 1 ]; then
        longEmerg "$(logInfo)" "Firmware Download Failed... Exiting"
        fw_api "${operid}" "Error" "Firmware_Download_Failed..."
        exit
    fi

    #Checking for Downloaded File
    if [ ! -f /tmp/firmware.tar.gz ]; then
        longCrit "$(logInfo)" "Image not Exists... Exiting"
        fw_api "${operid}" "Error" "Image_not_Exists"
        exit
    fi

    dwnld_md5sum=`md5sum /tmp/firmware.tar.gz |awk '{print$1}'`
    if [ $md5sum != $dwnld_md5sum ]; then
        longErr "$(logInfo)" "MD5SUM Mismatched-Firmware Upgrade Failed... Exiting..."
        fw_api "${operid}" "Error" "Error_with_Downloaded_Image_File"
        exit
    fi

    #Extracting the File
    rm /tmp/*.img > /dev/null 2>&1
    longNotice "$(logInfo)" "Extracting the Firmware Image"
    tar -xzvf /tmp/firmware.tar.gz -C /tmp/ > /dev/null 2>&1

    #Checking for Extracted File
    if [ ! -f /tmp/vchimay*.img ]; then
        longErr "$(logInfo)" "Image not Exists in tar file"
        fw_api "${operid}" "Error" "Image_not_Exists_in_tar_file"
        exit
    fi

    #Upgrading Image in Device
    longInfo "$(logInfo)" "Firmware upgrade in Progress"
    fw_api "${operid}" "Info" "Firmware_Upgrade_in_Progress"
    sysupgrade /tmp/vchimay*.img
    if [ "$?" -eq 1 ]; then
        longEmerg "$(logInfo)" "Firmware upgrade Failed..."
        fw_api "${operid}" "Error" "Firmware_Upgrade_Failed..."
    fi
}

keyFile="/opt/inseego/etc/key"
certFile="/etc/ssl/cert.pem"

#Generating the access Token
longInfo "$(logInfo)" "Generating Access Tokens...!!"
/opt/inseego/bin/accessToken.sh
accessKey=`jq -r .access $keyFile`
if [ -z "$accessKey" -o "$accessKey" = "null" ]
then
    longErr "$(logInfo)" "AccessKey is empty - $accessKey Exiting..."
    exit
fi
Key="Bearer ${accessKey}"

#Getting the mgmtIP and port details from ztp Configuration file
mgmtProto=`uci get ztp.EM_details.proto`
mgmtUrl=`uci get ztp.EM_details.domain`
mgmtPort=`uci get ztp.EM_details.port`

upgrade $@
