#!/bin/bash

#Fix for no environment issue when calling from mwan3
export PATH=/usr/sbin:/usr/bin:/sbin:/bin

#******************************************************************************#
# @file  : longqos.sh
# @brief : This file contains source code for longship Quality of
#          Service Feature [ Queuing,Shaping,Classification - WAN,LAN,
#          TUNNEL EGRESS | Ratelimiting,Policing - LAN INGRESS ]
#******************************************************************************#

[ -e /lib/functions/network.sh ] && . /lib/functions/network.sh

. /lib/functions.sh "QoS"

#Fetch the Caller Function name and Line number
logInfo() { echo "("${FUNCNAME[1]}":"${BASH_LINENO}")"; }

declare QueueNames
declare RateLimitNames
declare -A QueueParams

#default Lan linkspeed in Kbps
defaultLanSpeed=`uci -q get longqos.qos.lanspeed`
defaultWanSpeed=`uci -q get longqos.qos.wanspeed`
#No of queue param
NumQueue=4
#wan policy info
wanPolicy=''

TC=/usr/sbin/tc
IPTABLES=/usr/sbin/iptables
SPEEDTEST=/usr/bin/speedtest-cli

arch=`uname -m`
if [ "$arch" == "aarch64" ];then
    IP=/usr/sbin/ip
elif [ "$arch" == "i686" ];then
    IP=/sbin/ip
else
    longCrit "$(logInfo)" "Architechture Unknown..."
    exit
fi

#******************************************************************************#
#module           : del_iptables_rules
#chain            : chain names in iptables
#Description      : delete existing iptable rules for the interface
#******************************************************************************#
del_iptables_rules() {
    local chain=$1
    chainName=`echo "$chain" | awk '{print $NF}'`
    longNotice "$(logInfo)" "Resetting Iptables Rule for $chainName"
    $IPTABLES -t mangle -D $chain
    $IPTABLES -t mangle -F $chainName
    $IPTABLES -t mangle -X $chainName
    longNotice "$(logInfo)" "Deleted the $chainName Chain..."
}

#******************************************************************************#
#module      : del_tc_rules
#intf        : interface name
#type        : traffic type
#Description : deletes queue rules and rate_limit rules for the interface
#******************************************************************************#
del_tc_rules() {
    local intf=$1
    local type=$2

    if [ $type == "egress" ];then
        longNotice "$(logInfo)" "Resetting Queueing Rule for $intf"
        $TC qdisc del dev $intf root
    else
        longNotice "$(logInfo)" "Resetting Ratelimiting Rule for $intf"
        $TC qdisc del dev $intf handle ffff: ingress
        $TC qdisc del dev ifb$intf root
        $IP link set dev ifb$intf down
        $IP link  del dev ifb$intf type ifb
    fi
}

#******************************************************************************#
#module            : apply_qos_filters
#intf              : interface name
#QueueParams[name] : unique name for section
#QueueParams[type] : type of Network configuration
#Description       : applying netfilter rules and filter forward rules
#******************************************************************************#
apply_qos_filters() {
    local intf=$1
    local ctrlTunport=`uci -q get network.wg0.listen_port`
    local dataTunports
    local TUN_PORTS

    longInfo "$(logInfo)" "Creating the QOS_${QueueParams[name]}_$intf Chain..."
    $IPTABLES -t mangle -N QOS_${QueueParams[name]}_$intf
    $IPTABLES -t mangle -A POSTROUTING -o $intf -g QOS_${QueueParams[name]}_$intf

    #Queue 1
    $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf -o $intf \
        -p tcp -m tcp --tcp-flags SYN,RST,ACK ACK -m length \
        --length 0:128 -j MARK --set-mark 0x1/0xff
    $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf -o $intf \
        -m tos --tos Minimize-Delay -j MARK --set-mark 0x1/0xff
    #Queue 2
    $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf -o $intf \
        -m dscp --dscp 36 -j MARK --set-mark 0x2/0xff
    $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf -o $intf \
        -m dscp --dscp 46 -j MARK --set-mark 0x2/0xff
    $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf -o $intf \
        -m dscp --dscp 48 -j MARK --set-mark 0x2/0xff
    $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf -o $intf \
        -p tcp -m multiport --ports 22,53 -j MARK --set-mark 0x2/0xff

    if [ "${QueueParams[type]}" == "wanConfig" ];then
        if [ -z "$ctrlTunport" ];then
            longCrit "$(logInfo)" "Failed to fetch the control tunnel port!!"
        else
            $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf -o \
                $intf -p udp -m multiport --ports $ctrlTunport \
                -j MARK --set-mark 0x2/0xff
        fi
    fi
    #Queue 3
    $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf -o $intf \
        -m dscp --dscp 8 -j MARK --set-mark 0x3/0xff

    if [ "${QueueParams[type]}" == "wanConfig" ];then
        local tunIntfs=`uci show network | grep -w "tunlink='$NWname'" \
            | grep -v "wg0" | cut -d . -f2`
        for tunIntf in $tunIntfs;do
            dataTunports+="$(uci -q get network.$tunIntf.listen_port),"
        done
        dataTunports=`echo "${dataTunports}" | sed 's/.$//'`

        if [ ! -z "$ctrlTunport" ];then
            [[ ! -z "$dataTunports" ]] && TUN_PORTS="$dataTunports,$ctrlTunport" \
                || TUN_PORTS="$ctrlTunport"
        else
            TUN_PORTS="$dataTunports"
        fi
    fi

    if [ ! -z "$TUN_PORTS" ];then
        $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf -o $intf \
            -p udp -m multiport ! --ports $TUN_PORTS -j MARK --set-mark 0x3/0xff
    else
        $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf -o $intf \
            -p udp -j MARK --set-mark 0x3/0xff
    fi

    #Queue 1
    $TC filter add dev $intf parent 1: prio 1 protocol ip handle 0x01/0xff \
        fw flowid 1:11
    #Queue 2
    $TC filter add dev $intf parent 1: prio 2 protocol ip handle 0x02/0xff \
        fw flowid 1:12
    #Queue 3
    $TC filter add dev $intf parent 1: prio 3 protocol ip handle 0x03/0xff \
        fw flowid 1:13

    if [ "${QueueParams[type]}" == "wanConfig" ];then
        if [ ! -z "$dataTunports" ];then
            #Queue 5 [Tunnel Queue]
            $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf \
                -o $intf -p udp -m multiport --ports $dataTunports \
                -j MARK --set-mark 0x5/0xff
            $TC filter add dev $intf parent 1: prio 0 protocol ip \
                handle 0x05/0xff fw flowid 1:15
        fi
        #Egress Packet Marking
        $IPTABLES -t mangle -A QOS_${QueueParams[name]}_$intf -o $intf \
            -p udp -m multiport --ports 123,53 -j DSCP --set-dscp-class AF42
    fi
}

#******************************************************************************#
#module                : create_queues
#intf                  : interface name
#QueueParams[speed]    : root queue Speed
#QueueParams[rate]     : queue Min Rate
#QueueParams[ceil]     : queue Ceil Rate
#QueueParams[priority] : queue Priority
#NumQueue              : no of queues need to serve
#Description           : creating queues with tc commands for Queuing
#                        QDISC-HTB
#******************************************************************************#
create_queues() {
    local intf=$1
    $TC qdisc add dev $intf root handle 1: htb default 14
    $TC class add dev $intf parent 1: classid 1:1 htb quantum 12000 \
        rate ${QueueParams[speed]}kbit ceil ${QueueParams[speed]}kbit

    for (( num=1; num<=$NumQueue; num++ )); do
        local queueid=$(( 10 + num ))

        if [ ${QueueParams[rate$num]} -ne 0 ];then
            $TC class add dev $intf parent 1:1 classid 1:${queueid} \
                htb quantum 12000 rate ${QueueParams[rate$num]}kbit \
                ceil ${QueueParams[ceil$num]}kbit prio ${QueueParams[priority$num]}
        fi
    done
}

#******************************************************************************#
#module      : apply_ratelimit_rules
#intf        : interface name
#limitspeed  : configured speed in kbps
#Description : create and applying ratelimiter with tc commands for Ratelimiting
#              QDISC-HTB
#******************************************************************************#
apply_ratelimit_rules() {
    local intf=$1
    local limitspeed=$2

    $IP link add name ifb$intf type ifb
    $TC qdisc add dev $intf handle ffff: ingress
    $TC qdisc add dev ifb$intf root handle 1: htb default 10
    $TC class add dev ifb$intf parent 1: classid 1:1 htb quantum 12000 \
        rate ${limitspeed}kbit ceil ${limitspeed}kbit
    $TC class add dev ifb$intf parent 1:1 classid 1:10 htb quantum 12000 \
        rate 100kbit ceil ${limitspeed}kbit prio 0
    $TC class add dev ifb$intf parent 1:1 classid 1:11 htb quantum 12000 \
        rate ${QueueParams[speed]}kbit ceil ${QueueParams[speed]}kbit prio 0

    $IP link set dev ifb$intf up

    $TC filter add dev $intf parent ffff: protocol all prio 10 u32 match \
        u32 0 0 flowid 1:1 action mirred egress redirect dev ifb$intf
    if [ -z ${QueueParams[srcIPs]} ];then
        QueueParams[srcIPs]='0.0.0.0/0'
    fi
    for ip in ${QueueParams[srcIPs]}; do
        $TC filter add dev ifb$intf parent 1:0 protocol ip prio 1 u32 \
            match ip src $ip flowid 1:11
    done
}

#******************************************************************************#
#module       : conv_percent
#speedPercent : configured percent value
#totalSpeed   : total speed based on speedtest
#Description  : manipulate the percent value from total speed
#Return       : percent value from totalspeed
#******************************************************************************#
conv_percent() {
    local speedPercent=$1
    local totalSpeed=$2
    echo $(($totalSpeed*$speedPercent/100))
}

#******************************************************************************#
#module                   : init_ratelimit_rules
#sectionName              : section name of the Configuration
#QueueParams[networkName] : root queue Speed for ratelimit
#Description              : fetch and convert the configured percent into values
#                           based on linkspeed and applying ratelimit rules
#******************************************************************************#
init_ratelimit_rules() {
    local sectionName=$1
    local _value
    local intf

    longInfo "$(logInfo)" "Applying Ratelimiting for $sectionName"
    for NWname in ${QueueParams[networkName]};do
        network_get_device intf $NWname
        if [ -z $intf ];then
            longErr "$(logInfo)" "Not able to find the interface in $NWname"
            continue
        fi
        if [ $(cat /sys/class/net/$intf/carrier) -eq 0 ];then
            longErr "$(logInfo)" "$intf Link not Detected...!"
            limitspeed=$defaultLanSpeed
            longNotice "$(logInfo)" "Configuring Default Linkspeed $limitspeed Kbps"
        elif [[ "$intf" == "br-"* ]];then
            limitspeed=$defaultLanSpeed
        else
            limitspeed=$(( 1000 * $(cat /sys/class/net/$intf/speed) ))
        fi
        config_get _value "$sectionName" "speed"
        longInfo "$(logInfo)" "Limit Percent $_value"
        QueueParams[speed]=$(conv_percent $_value $limitspeed)
        longInfo "$(logInfo)" "Ratelimitspeed ${QueueParams[speed]} Kbps"
        if [ ! -z $intf ]; then
            tc qdisc show dev ifb$intf 2>/dev/null | grep htb 1>/dev/null
            if [ `echo $?` == 0 ];then
                del_tc_rules "$intf" "ingress"
            fi
            longInfo "$(logInfo)" "Applying Ratelimiting Rules in $NWname - $intf"
            apply_ratelimit_rules $intf $limitspeed
        else
            longErr "$(logInfo)" "Not able to find the NetworkName - $NWname..."
        fi
    done
}
#******************************************************************************#
#module       : wanspeedtest
#Description  : test the wan uplink speed with public servers based
#               on specific or active wan.
#NwName       : Network Name
#SrcHost      : IP binded with the interface.
#Return value : linkspeed of wan interface.
#******************************************************************************#
wanspeedtest() {
    local NwName=$1
    local profID=$2
    local SrcHost=$3
    local linkspeed=0
    longNotice "$(logInfo)" "Starting Speedtest for $NwName"
    if [ `uci -q get longqos.$profID.speedtest` == 1 ]; then
        if [ -z "$SrcHost" ];then
            linkspeed=`$SPEEDTEST --json --no-download | jq .upload`
        else
            linkspeed=`$SPEEDTEST --json --no-download --source $SrcHost | jq .upload`
        fi
        linkspeed=`echo $linkspeed | awk '{ byte =$1/1024 ; print int(byte) }'`
    else [ `uci -q get longqos.$profID.wanBandwidth` ]
        linkspeed=`uci -q get longqos.$profID.wanBandwidth`
    fi
    if [ -z $linkspeed ] || [ $linkspeed -eq 0 ]; then
        longInfo "$(logInfo)" "Configuring Linkspeed"
        linkspeed=$defaultWanSpeed
        longInfo "$(logInfo)" "Configuring Linkspeed as $linkspeed Kbps"
    fi
    echo $linkspeed
}

#******************************************************************************#
#module                   : init_queue_rules
#sectionName              : section name of the Configuration
#QueueParams[networkName] : root queue Speed for ratelimit
#Description              : fetch and convert the percent into values based on
#                           speedtest/linkspeed and start queuing process
#******************************************************************************#
init_queue_rules() {
    local sectionName=$1
    local _value
    local linkspeed=0
    local wanLink=0
    local wanSection=''
    local activeWan=''
    local profID=''

    longInfo "$(logInfo)" "Initiate Queueing for $sectionName"
    for NWname in ${QueueParams[networkName]};do
        profID=`uci show longqos | grep -w $NWname | cut -d . -f2`
        if [ "${QueueParams[type]}" == "wanConfig" ];then
            network_get_device intf $NWname
            if [ -z "$intf" ];then
                longErr "$(logInfo)" "Not able to find the interface in $NWname"
                continue
            fi
            if [ "$wanPolicy" == "balanced" ];then
                SrcHost=`ifconfig $intf | awk '/inet addr:/ {print $2}'\
                    | cut -d : -f2`
                if [ -z "$SrcHost" ];then
                    longErr "$(logInfo)" "Not able fetch $intf IP Address"\
                        "Rejeting QoS Rule for $NWname"
                    continue
                fi
                #SrcHost was used to bind the interface when doing speedtest
                linkspeed=$(wanspeedtest $NWname $profID $SrcHost)
            else
                activeWan=`mwan3 policies | grep -A 1 "$wanPolicy" \
                    | head -2 | tail -1 | cut -d ' ' -f 2`
                if [ "$activeWan" == "$NWname" ];then
                    linkspeed=$(wanspeedtest $NWname $profID)
                else
                    longWarn "$(logInfo)" "The Network was not the Active Wan."\
                        "Rejeting QoS Rule for $NWname"
                    continue
                fi
            fi
            longInfo "$(logInfo)" "$NWname - Linkspeed $linkspeed Kbps"
        elif [ "${QueueParams[type]}" == "lanConfig" ];then
            network_get_device intf $NWname
            if [ -z $intf ];then
                longErr "$(logInfo)" "Not able to fetch interface for $NWname"
                continue
            fi
            if [ $(cat /sys/class/net/$intf/carrier) -eq 0 ];then
                longErr "$(logInfo)" "$intf Link is not Detected...!"
                linkspeed=$defaultLanSpeed
                longInfo "$(logInfo)" "Configuring Default Linkspeed $linkspeed Kbps"
            elif [[ "$intf" == "br-"* ]];then
                linkspeed=$defaultLanSpeed
            else
                linkspeed=$(( 1000 * $(cat /sys/class/net/$intf/speed) ))
            fi
        elif [ "${QueueParams[type]}" == "tunnelConfig" ];then
            intf=$NWname
            wanLink=`uci -q get network.$intf.tunlink`
            if [ -z $wanLink ];then
                longErr "$(logInfo)" "Not able to find wan_link for $intf"
                continue
            fi
            wanIntf=`uci -q get network.$wanLink.ifname`
            wanLinkSpeed=`uci -q get longqos.qos.$wanIntf\_speed`
            wanSection=`uci show longqos | grep -w $wanLink | \
                grep "networkName" | cut -d . -f2`
            #fetching configured WAN uplink percentage
            config_get _value "$wanSection" "speed"
            limitedWanLinkspeed=$(conv_percent $_value $wanLinkSpeed)
            #Updating tunnel uplink rate based on wan queue-5 ceil rate
            config_get _value "$wanSection" "ceil5"
            linkspeed=$(conv_percent $_value $limitedWanLinkspeed)
            longDebug "$(logInfo)" "Tunnel Params wanLink=$wanLink" \
                "wanLinkSpeed=$wanLinkSpeed limitedWanLinkspeed=$limitedWanLinkspeed" \
                "wanSection=$wanSection Tun_linkspeed=$linkspeed"
        fi
        if [ -z $intf ]; then
            longErr "$(logInfo)" "Not able to find the interface in $NWname"
            continue
        fi
        intfTemp=`echo "$intf" | sed -E 's/[^[:alnum:]]+/_/g'`
        uci -q set longqos.qos.$intfTemp\_speed=$linkspeed
        longInfo "$(logInfo)" "Linkspeed $linkspeed Kbps"
        config_get _value "$sectionName" "speed"
        longInfo "$(logInfo)" "Limit Percent $_value"
        QueueParams[speed]=$(conv_percent $_value $linkspeed)
        longInfo "$(logInfo)" "Queuespeed ${QueueParams[speed]} kbps"
        if [ "${QueueParams[speed]}" != 0 ];then
            longInfo "$(logInfo)" "Fetching Queue Configuration for" \
                "Network-$NWname Interface-$intf"
            for (( num=1; num<=$NumQueue; num++ ));do
                config_get _value "$sectionName" "rate$num"
                longDebug "$(logInfo)" "Rate$num Percent $_value"
                QueueParams[rate$num]=$(conv_percent $_value ${QueueParams[speed]})
                config_get _value "$sectionName" "ceil$num"
                longDebug "$(logInfo)" "Ceil$num Percent $_value"
                QueueParams[ceil$num]=$(conv_percent $_value ${QueueParams[speed]})
                longDebug "$(logInfo)" "Rate$num value ${QueueParams[rate$num]} kbps" \
                    "Ceil$num value ${QueueParams[ceil$num]} kbps"
            done
            if [ ! -z $intf ]; then
                $IPTABLES -t mangle -S POSTROUTING | grep "QOS_${QueueParams[name]}_$intf" 1>/dev/null
                if [ `echo $?` == 0 ];then
                    local chain="POSTROUTING -o $intf -g QOS_${QueueParams[name]}_$intf"
                    longInfo "$(logInfo)" "Resetting Rules in $NWname - $intf"
                    del_iptables_rules "$chain"
                fi
                tc qdisc show dev $intf 2>/dev/null | grep htb 1>/dev/null
                if [ `echo $?` == 0 ];then
                    del_tc_rules "$intf" "egress"
                fi
                longInfo "$(logInfo)" "Applying Queuing Rule in $NWname - $intf"
                create_queues $intf
                longInfo "$(logInfo)" "Applying Filter Rule in $NWname - $intf"
                apply_qos_filters $intf
            fi
        else
            longInfo "$(logInfo)" "Not able to apply the Queueing rules as wanspeed is ${QueueParams[speed]} kbps"
        fi

    done
}

#******************************************************************************#
#module       : get_ratelimit_config
#sectionName  : section name of the Configuration
#Description  : fetch the rate limit parameters from configuration file
#******************************************************************************#
get_ratelimit_config() {
    local sectionName=$1
    local _value
    QueueParams=()

    longInfo "$(logInfo)" "Fetching Ratelimit Configuration Parameters in $sectionName"
    config_get _value "$sectionName" "type"
    QueueParams[type]=$_value
    config_get _value "$sectionName" "name"
    QueueParams[name]=$_value
    config_get _value "$sectionName" "networkName"
    QueueParams[networkName]=$_value
    config_get _value "$sectionName" "speed"
    QueueParams[speed]=$_value
    config_get _value "$sectionName" "srcIPs"
    QueueParams[srcIPs]=$_value

    #TODO:Input validation for QueueParams
    longDebug "$(logInfo)" "Queuing Configuration Parameters ${QueueParams[@]}"
}

#******************************************************************************#
#module       : get_queue_config
#sectionName  : Section name of the Configuration
#Description  : fetch the queueing parameters from configuration file
#******************************************************************************#
get_queue_config() {
    local sectionName=$1
    local _value
    QueueParams=()

    longInfo "$(logInfo)" "Fetching Queue Configuration Parameters in $sectionName"
    config_get _value "$sectionName" "name"
    QueueParams[name]=$_value
    config_get _value "$sectionName" "networkName"
    QueueParams[networkName]=$_value
    config_get _value "$sectionName" "speed"
    QueueParams[speed]=$_value
    config_get _value "$sectionName" "type"
    QueueParams[type]=$_value
    if [ "${QueueParams[type]}" == "wanConfig" ];then
        NumQueue=5
        config_get _value "$sectionName" "setmark"
        QueueParams[setmark]=$_value
    else
        NumQueue=4
    fi
    longInfo "$(logInfo)" "Number of Queue=$NumQueue in $sectionName"
    for (( num=1; num<=$NumQueue; num++ )); do
        config_get _value "$sectionName" "rate$num"
        QueueParams[rate$num]=$_value
        config_get _value "$sectionName" "ceil$num"
        QueueParams[ceil$num]=$_value
        config_get _value "$sectionName" "priority$num"
        QueueParams[priority$num]=$_value
        config_get _value "$sectionName" "config$num"
        QueueParams[config$num]=$_value
    done
    #TODO:Input validation for QueueParams
    longDebug "$(logInfo)" "Queuing Configuration Parameters ${QueueParams[@]}"
}

#******************************************************************************#
#module      : delete_qos_rules
#Description : stoping Qos Module
#******************************************************************************#
delete_qos_rules() {
    longInfo "$(logInfo)" "Removing QoS"
    tc qdisc | grep htb | awk '{print $5}' |
    {
        while read intf;
        do
        if [ ! -z "$intf" ];then
            if [[ "$intf" == "ifb"* ]]; then
                intf=`echo $intf | sed -e "s/^ifb//"`
                del_tc_rules "$intf" "ingress"
            else
                del_tc_rules "$intf" "egress"
            fi
        else
            longInfo "$(logInfo)" "No existing tc rules found"
        fi
        done
    }
    iptables -t mangle -S POSTROUTING | grep QOS | cut -d " " -f2- |
    {
    while read chain;
    do
        if [ ! -z "$chain" ];
        then
            del_iptables_rules "$chain"
        else
            longInfo "$(logInfo)" "No existing Iptable rules found"
        fi
    done
    }

    grep "longqos" /etc/crontabs/root
    ret=`echo $?`
    if [ $ret -eq 0 ]; then
        longInfo "$(logInfo)" "Stopping QoS Cronjob activity"
        sed -i '/longqos/d' /etc/crontabs/root
        /etc/init.d/cron restart
    fi
}

#******************************************************************************#
#module      : init_qos
#Description : starting Qos Module
#******************************************************************************#
init_qos() {
    local sectionName
    local enabled=0

    enabled=`uci -q get longqos.qos.enabled`
    if [ $enabled -ne 1 ];then
        longErr "$(logInfo)" "QoS not enabled...!!"
        exit
    fi

    cronTime=`uci -q get longqos.qos.speedtestdur`
    if [ -z $cronTime ] || [ $cronTime -eq 0 ]; then
        longErr "$(logInfo)" "Cronjob value not configured Properly!!!" \
            "Configuring Default Duration to 1 Hours"
        cronTime=1
    fi

    grep "longqos" /etc/crontabs/root
    rett=`echo $?`
    if [ $rett -eq 0 ]; then
        longInfo "$(logInfo)" "Stopping Qos Cronjob activity"
        sed -i '/longqos/d' /etc/crontabs/root
    fi
    longInfo "$(logInfo)" "Initiating QoS Cronjob for $cronTime Hours"
    echo "*/* $cronTime * * * bash /opt/inseego/bin/longqos.sh restart" >> /etc/crontabs/root
    /etc/init.d/cron restart

    if [ ! -z "$QueueNames" ];then
        wanPolicy=`uci -q get mwan3.default_rule.use_policy`
        longDebug "$(logInfo)" "Queueing Section" \
            "Names - $QueueNames"
        for sectionName in $QueueNames; do
            config_get type "$sectionName" "type"
            longInfo "$(logInfo)" "Fetching $type Configuration in $sectionName"
            if [ "$type" == "wanConfig" ] || [ "$type" == "lanConfig" ] \
                || [ "$type" == "tunnelConfig" ];then
                get_queue_config $sectionName
                init_queue_rules $sectionName
                #Linkspeed updation in the config file
                uci commit longqos
            fi
        done
    fi
    if [ ! -z "$RateLimitNames" ];then
        longDebug "$(logInfo)" "Ratelimiting Section Names - $RateLimitNames"
        for sectionName in $RateLimitNames;do
            config_get type "$sectionName" "type"
            longInfo "$(logInfo)" "Fetching $type Configuration in $sectionName"
            if [ "$type" == "lanRateLimit" ];then
                get_ratelimit_config $sectionName
                init_ratelimit_rules $sectionName
        fi
    done
    fi
}

#******************************************************************************#
#module      : qos_stat
#Description : Collects the Queue statistics and store in the tmp directory
#******************************************************************************#
qos_stat() {
    case "$1" in
        start)
            longInfo "$(logInfo)" "Starting QoS Statistics"
            /etc/init.d/qosstat start
        ;;
        stop)
            longInfo "$(logInfo)" "Stoping QoS Statistics"
            /etc/init.d/qosstat stop 2>/dev/null
        ;;
    esac
}

#******************************************************************************#
#module      : config_cb
#Description : fetch all the section names from configuration file
#******************************************************************************#
config_cb() {
    local type="$1"
    local name="$2"
    if [ "$type" == "queueconfig" ];then
        append QueueNames "$name"
    elif [ "$type" == "ratelimitconfig" ];then
        append RateLimitNames "$name"
    fi
}

lockFile="/tmp/qos.lock"

if [[ ! -e "$lockFile" ]]; then
    echo $$ > $lockFile
else
    longCrit "$(logInfo)" "QoS is Already Running"
    longEmerg "$(logInfo)" "Killing Existing Process- $(cat /tmp/qos.lock)  New Process to run- $$"
    kill -9 $(cat /tmp/qos.lock)
    echo $$ > $lockFile
fi
trap 'rm -rf "$lockFile"' 0

config_load longqos
QueueNames=$(echo "$QueueNames" | tr ' ' '\n' | sort -r | tr '\n' ' ')

case "$1" in
    start)
        longInfo "$(logInfo)" "Starting QoS"
        init_qos
        qos_stat start
    ;;
    stop)
        longInfo "$(logInfo)" "Stoping QoS"
        delete_qos_rules
        qos_stat stop
    ;;
    restart)
        longInfo "$(logInfo)" "Restarting QoS"
        delete_qos_rules
        qos_stat stop
        init_qos
        qos_stat start
    ;;
esac
