#!/bin/bash
#******************************************************************************#
# @file          : KeyRot.sh
# @brief         : This script is responsible for performing key rotation
#                  configuration for control and data tunnel.
#******************************************************************************#

. /lib/functions.sh "TUNNEL"
. /opt/inseego/bin/sdedge_common_params.sh

#Fetch the Caller Function name and Line number
logInfo() { echo "("${FUNCNAME[1]}":"${BASH_LINENO}")"; }

#******************************************************************************#
#module       : curlExec
#arg1         : API need to be called
#arg2         : Payload for API
#Description  : Used to call the API
#Return       : API response
#******************************************************************************#
curlExec()
{
    local apiName=$1
    local datarawParam=$2
    local apiResp=""
    local curlResp=""

    apiResp=$(curl --cacert $certFile $allowK --location \
	    --connect-timeout $timeout -m $maxtimeout --request POST \
	    $mgmtUrl:$mgmtPort/api-device/device/$apiName \
	    --header  'Authorization: '"$Key"'' --header \
	    'Content-Type: application/json' --data-raw $datarawParam --header "X-Tenant-Name: $tenName")
    curlResp="$?"
    longDebug "$(logInfo)" "Curl Exit code for $apiName API - $curlResp"
    longDebug "$(logInfo)" "$apiName Response: $apiResp"
    echo $apiResp
}

#******************************************************************************#
#module       : Usage
#Description  : Prints the script usage
#******************************************************************************#
Usage()
{
    echo "Usage: KeyRot.sh [TunnelName] [PresharedKey] [OperationID]"
    echo -e "\tTunnelName     : Wireguard configuration peer name"
    echo -e "\tPresharedKey   : Wireguard Preshared-key"
    echo -e "\tOperationID    : OperationID while executing key rotation operation"
    exit
}

#******************************************************************************#
# @function      : loginAPI
# @param         : (none)
# @brief         : This function is used for API Authorization login.
#******************************************************************************#
loginAPI() {
    longInfo "$(logInfo)" "Generating the Tokens..!!"
    /opt/inseego/bin/access.sh
    accessKey=`jq -r .access $keyFile`
    if [ -z "$accessKey" -o "$accessKey" = "null" ]
    then
        longErr "$(logInfo)" "Login API response failed: AccessKey - $accessKey"
        exit
    fi
    Key="Bearer ${accessKey}"
    longDebug "$(logInfo)" "AccessKey - $accessKey"
    echo $Key
}

#******************************************************************************#
#module       : configurePSK
#Description  : Configure the preshared key to wireguard peer.
#******************************************************************************#
configurePSK() {
    longInfo "$(logInfo)" "Key Rotation - Configuration Inprogress..."
    longDebug "$(logInfo)" "PresharedKey: $psk"
    interface=`uci -q get network.$tunnel_name | cut -d '_' -f2`
    peer=`uci -q get network.$tunnel_name.public_key`
    prev_psk=`uci -q get network.$tunnel_name.preshared_key`
    uci set network.$tunnel_name.preshared_key=$psk
    uci commit network
    echo $psk > /tmp/"$interface"_psk
    wg set $interface peer $peer preshared-key /tmp/"$interface"_psk
    rm -f /tmp/"$interface"_psk
}

if [ $# != 3 ]; then
    longWarn "$(logInfo)" "Insufficient Arguments!!"
    Usage
fi

tunnel_name=$1
psk=$2
opr_id=$3
keyFile="/opt/inseego/etc/key"
certFile="/etc/ssl/cert.pem"

if [[ $tunnel_name == "controlTunnel" ]]; then
    #Call loginAPI to generate access token key
    loginAPI
    #Call configurePSK to configure preshared key in device
    configurePSK

    retryCount=0
    [[ `wg show wg0 preshared-keys | awk '{print $2}'` == $psk ]] \
        && operStatus="successful" || operStatus="failed"

    while :
    do
        if [ "$retryCount" -le 3 ]; then
            apiResp=$(curlExec "controlTunnelKeyRotationStatus" '{"operation_uuid":"'$opr_id'","operation_status":"'$operStatus'","remarks":"Configuration_'$operStatus'","apiKey":"'$apiKey'"}')
            [[ -z "$apiResp" ]] || break
            sleep 5
            retryCount=`expr $retryCount + 1`
            [[ $retryCount -le 3 ]] && \
                longCrit "$(logInfo)" "Keyrotation API response empty:" \
                    "Retrying... count-$retryCount"
        else
            longCrit "$(logInfo)" "Key rotation API response failed..."
            break
        fi
    done

    resp=`echo $apiResp | jq -r .Success`
    longInfo "$(logInfo)" "Key Rotation Response - $resp"
    if [[ $resp != "true" ]]; then
        longInfo "$(logInfo)" "Restoring Previous Configuration"
        uci set network.$tunnel_name.preshared_key=$prev_psk
        uci commit network
        echo $prev_psk > /tmp/"$interface"_psk
        wg set $interface peer $peer preshared-key /tmp/"$interface"_psk
        rm -f /tmp/"$interface"_psk
    fi
fi
