#!/bin/bash

#******************************************************************************#
# @file          : data_tunnel_switch.sh
# @brief         : This script is responsible for Data Tunnel switching w.r.t
#                  mwan3 configuration.
#******************************************************************************#

. /lib/functions.sh "TUNNEL"

#Fetch the Caller Function name and Line number
logInfo() { echo "("${FUNCNAME[1]}":"${BASH_LINENO}")"; }

#For the existence of interface wg<1,2,...n> on device
check_wg=`uci -q get network.datatunnel_list.usemember`
#Checks whether data tunnel is there in device
if [[ -z "$check_wg" ]]
then
	longNotice "$(logInfo)" "Exiting the data tunnel switch script when no data tunnel is available"
	exit
fi

LB_flag=$(if [[ $(uci -q get mwan3.default_rule.use_policy) == "balanced" ]]; then echo 1; else echo 0; fi)
if [[ $LB_flag == 1 ]]
then
	longNotice "$(logInfo)" "Data Tunnel switching will not happen,hence WAN LoadBalancing is enabled"
fi

#******************************************************************************#
# @function     : add_route
# @param        : wan_interface,endpoint_IP
# @brief        : This Function will add a static route for the endpoint_IP with
#                 the gateway as selected wan_interface
#******************************************************************************#
add_route()
{
	binding_wan=$1
	endpoint_IP=$2
	ip route del $endpoint_IP
	#Condition to choose the interface name based on the wan type
	if [[ `uci -q get network.$binding_wan.type` == "bridge" ]]
	then
		interface=br-$binding_wan
	else
		interface=`uci -q get network.$binding_wan.ifname`
	fi
	gateway=`ip route | grep default | grep $interface | awk {'print $3'}`
	metric=`uci -q get network.$binding_wan.metric`
	ip route add "$endpoint_IP" via $gateway dev $interface metric $metric
}

active_WAN=$1
htplug_event=$2
htplug_interface=$3
check_WAN=${active_WAN:0:3}
#Loop which interates the available tunnel interfaces
for element in `uci -q get network.datatunnel_list.usemember`
do
	endpoint_data_IP=`uci -q get network."${element}".endpoint_host`
	wg_Iface=`uci -q get network."${element}" | cut -d '_' -f2`

	if [[ $LB_flag == 1 ]] && [[ -n `uci -q get network.tunlink_iface.${wg_Iface}` ]]
	then
		#Fetch the mapped wan interface
		tunlink_iface=`uci -q get network.tunlink_iface.${wg_Iface}`
		#When nohostroute option to 1,Wireguard will not automatically add route with mapped tunlink interface
		uci set network.${wg_Iface}.nohostroute="1"
		uci set network.${wg_Iface}.tunlink=${tunlink_iface}
		uci commit network
		longInfo "$(logInfo)" "${wg_Iface} is mapped with ${tunlink_iface}"
		#Condition to choose the interface based on the active_wan type
		if [ `uci -q get network.$tunlink_iface.type` == "bridge" ]
		then
			interface=br-$tunlink_iface
		else
			interface=`uci -q get network.$tunlink_iface.ifname`
		fi
		longDebug "$(logInfo)" "Event - $htplug_event : Interface - $htplug_interface"
		if [[ $htplug_interface == $tunlink_iface && $htplug_event == "disconnected" ]]
		then
			#Down the wireguard interface when the respective mapped interface is down
			ifdown $wg_Iface
			longEmerg "$(logInfo)" "${interface} link is down. so, Making ${wg_Iface} link to down"
		elif [[ -z `ifconfig $wg_Iface` && $htplug_event == "connected" && $htplug_interface == $tunlink_iface ]]
		then
			ifup $wg_Iface
			longNotice "$(logInfo)" "Making ${wg_Iface} link to up"
		fi
		if [[ -n $tunlink_iface ]] && [[ -n $endpoint_data_IP ]]; then
			add_route ${tunlink_iface} ${endpoint_data_IP}
		else
			longErr "$(logInfo)" "Tunlink Interface/Endpoint IP is empty for $wg_Iface!!"
		fi
	#Check for the active_wan having "wan" as it would return value as wan,wan_1G and so on
	elif [[ $check_WAN == "wan" && $LB_flag == 0 ]]
	then
		longNotice "$(logInfo)" "The active WAN is $active_WAN"
		#when wg interface is made down in loadbalancing case, making it up here
		if [[ -z `ifconfig $wg_Iface` ]]
		then
			ifup $wg_Iface
			longNotice "$(logInfo)" "Making ${wg_Iface} link to up"
		fi
		uci set network.${wg_Iface}.nohostroute='1'
		uci set network.${wg_Iface}.tunlink="$active_WAN"
		uci commit network
		if [[ -n $active_WAN ]] && [[ -n $endpoint_data_IP ]]; then
			add_route ${active_WAN} ${endpoint_data_IP}
		else
			longErr "$(logInfo)" "Tunlink Interface/Endpoint IP is empty for $wg_Iface!!"
		fi
	#Check for the active_WAN returned as "unreachable" by mwan3
	elif [[ $check_WAN == "unr" && $LB_flag == 0 ]]
	then
		longNotice "$(logInfo)" "The active WAN is $active_WAN"
		uci set network.${wg_Iface}.nohostroute="1"
		uci set network.${wg_Iface}.tunlink=""
		uci commit network
	fi
done
