#!/bin/bash

#******************************************************************************#
# @file          : exe_ztp.sh
# @brief         : To perform ZTP initial handshake with EM.
#******************************************************************************#

. /lib/functions.sh "ZTP"

#Fetch the Caller Function name and Line number
logInfo() { echo "("${FUNCNAME[1]}":"${BASH_LINENO}")"; }

#******************************************************************************#
#module       : curlExec
#arg1         : API need to be called
#arg2         : Payload for API
#Description  : Used to call the API
#Return       : API response
#******************************************************************************#
curlExec()
{
    local apiName=$1
    local datarawParam=$2
    local apiResp=""
    local curlResp=""

    apiResp=$(curl --cacert $certFile --location \
	    --connect-timeout 60 -m 60 --request POST \
	    $mgmtProto://$mgmtUrl:$mgmtPort/api-device/device/$apiName \
	    --header  'Authorization: '"$Key"'' --header \
	    'Content-Type: application/json' --data-raw $datarawParam \
	    -w "HTTP_STATUS_CODE: %{http_code}")
    curlResp="$?"
    longDebug "$(logInfo)" "Curl Exit code for $apiName API - $curlResp"
    longDebug "$(logInfo)" "$apiName Response: $apiResp"
    echo $apiResp
}

#******************************************************************************#
#module       : encData
#arg1         : device username or password
#Description  : encrypt the data with aes-256-cbc
#Return       : encrypted username or password
#******************************************************************************#
encData()
{
    local data=$1
    encdata=`echo $data | openssl aes-256-cbc -a -salt -pass pass:$md5sum -pbkdf2`
    echo $encdata | tr -d ' '
}

#******************************************************************************#
#module       : decData
#arg1         : encrypted username or password
#Description  : decrypt the data with aes-256-cbc
#Return       : decrypted username or password
#******************************************************************************#
decData()
{
    local data=$1
    decdata=`echo $data | openssl aes-256-cbc -d -a -pass pass:$md5sum -pbkdf2`
    echo $decdata
}

#******************************************************************************#
#module       : removeZTPfiles
#Description  : Removing ZTP Generated files
#Return       : None
#******************************************************************************#
removeZTPfiles()
{
    rm -rf /opt/inseego/etc/device_details > /dev/null 2>&1
}

#******************************************************************************#
#module       : genAccesskey
#Description  : generate accesskey token
#Return       : accesskey
#******************************************************************************#
genAccesskey()
{
    longNotice "$(logInfo)" "Generating the Tokens..!!"
    /opt/inseego/bin/access.sh
    accessKey=`jq -r .access $keyFile`
    if [ -z "$accessKey" -o "$accessKey" = "null" ]
    then
        longErr "$(logInfo)" "AccessKey is empty - Removing ZTP files and Exiting..."
        removeZTPfiles
        exit
    fi
    Key="Bearer ${accessKey}"
}

#******************************************************************************#
#module       : configureZabbixAgent
#arg1         : EM serverIP
#arg2         : device hostname
#Description  : configuring zabbix agent
#******************************************************************************#
configureZabbixAgent()
{
    serverIP=$1
    hostName=$2

    sed -i 's/Server=.*/Server='$serverIP'/' $zabbixConf
    sed -i 's/Hostname=.*/Hostname='$hostName'/' $zabbixConf
    sed -i 's/ServerActive=.*/ServerActive='$serverIP'/' $zabbixConf

    /etc/init.d/zabbix_agentd restart
}

#******************************************************************************#
#module       : updatePasswd
#arg1         : password
#Description  : Updating password
#******************************************************************************#
updatePasswd()
{
    local password=$1
    echo $userName:$password | chpasswd
    if [ "$?" -eq 1 ];then
        longEmerg "$(logInfo)" "Password Update Failed!!!"
        echo "false"
    else
        longNotice "$(logInfo)" "Password Update Successfull.."
        echo "true"
    fi
}

#******************************************************************************#
#module       : ztp_default_template_api
#arg1         : factory operation ID
#Description  : Applying Default template operation
#******************************************************************************#
ztp_default_template_api()
{
    factoryoperid=$1
    local retryCount=0
    local resp=1

    longInfo "$(logInfo)" "Waiting for Device Reachability in EM.."
    while [ $resp -eq 1 ]
    do
        longInfo "$(logInfo)" "Waiting for Tunnnel Reachability..!!"
        ping -c1 `uci -q get network.controlTunnel.allowed_ips | cut -d " " -f1 | tr -d "'"`
        resp=`echo $?`
        sleep 1
    done
    sleep 45
    device_mac=`echo $deviceMac | tr '[A-Z]' '[a-z]'`
    longDebug "$(logInfo)" "Apply Template API Param -" \
        "DeviceMAC: $device_mac Operation ID: $factoryoperid"

    while :
    do
        if [ "$retryCount" -le 3 ]; then
            apiResp=$(curlExec "applyDefaultZTPconfig" '{"device_mac":"'$device_mac'","operation_id":"'$factoryoperid'"}')
            [[ -z "$apiResp" ]] || break
            sleep 5
            retryCount=`expr $retryCount + 1`
            [[ "$retryCount" -le 3 ]] && \
                longCrit "$(logInfo)" "ZTP Default Template API response empty:" \
                    "Retrying... count-$retryCount"
        else
            longCrit "$(logInfo)" "ZTP Default Template API response failed..."
            exit
        fi
    done

    echo $apiResp | jq '.messages[0] .message' | grep "Token is invalid or expired"
    jresp=`echo $?`
    if [ "$jresp" -eq 0 ];then
         longNotice "$(logInfo)" "Access token Expired -" \
             "Calling Bootstrap template API by generating new token"
         genAccesskey
         ztp_default_template_api $factoryoperid
    fi
}

#******************************************************************************#
#module       : sendDeviceInfo
#Description  : Seding device information to EM
#******************************************************************************#
sendDeviceInfo()
{
    local retryCount=0

    longCrit "$(logInfo)" "Generating Password..."
    userName="root"
    encUserName=$(encData $userName)
    password=`cat /dev/urandom | env LC_CTYPE=C tr -dc _ABCDEFGHJKLMNPQRSTUVWXYZabcdefghjklmnpqrstuvwxyz23456789- | head -c 20; echo;`
    if [ -z "$password" ];then
        longEmerg "$(logInfo)" "Password Generation Failed!!! Resetting Default Password..."
        password="1nSeeG0C!0uDR0u1eR@7"
    fi
    longInfo "$(logInfo)" "Updating Password..."
    resp=$(updatePasswd $password)
    if [ "$resp" == "false" ];then
        longErr "$(logInfo)" "Password Updation Failed!!! Resetting Default Password..."
        password="1nSeeG0C!0uDR0u1eR@7"
    fi
    longInfo "$(logInfo)" "Encoding Password..."
    encPassword=`echo "$(encData $password)"| tee /opt/inseego/etc/password`

    longInfo "$(logInfo)" "Send Info to Edge Manager..!!"
    longDebug "$(logInfo)" "Device Details...\n Tunnel IP: $tunnelIP" \
        "Username: $userName encPassword: $encPassword DeviceMac: $deviceMac"

    while :
    do
        if [ "$retryCount" -le 6 ]; then
            apiResp=$(curlExec "init" '{"device_mac":"'$deviceMac'","device_username":"'$encUserName'","device_ctunnel_ip":"'$tunnelIP'","device_password":"'$encPassword'"}')
            [[ -z "$apiResp" ]] || { [[ ! -z "$apiResp" ]] && [[ $(echo "$apiResp" | awk -F 'HTTP_STATUS_CODE: ' '{print $2}') != "200" ]]; } || { [[ $(echo "$apiResp" | jq -r .success) == "false" ]] && [[ $(echo "$apiResp" | jq -r .message) == "InProgress" ]]; } || break
            sleep 20
            retryCount=`expr $retryCount + 1`
            [[ "$retryCount" -le 6 ]] && \
                longCrit "$(logInfo)" "Init API response empty:" \
                    "Retrying... count-$retryCount"
        else
            longCrit "$(logInfo)" "Init API response failed..."
            exit
        fi
   done

   resp=`echo $apiResp | jq -r .success`
   msgResp=`echo $apiResp | jq -r .message`
   if [ "$resp" == "false" ] && [ "$msgResp" == "Failed" ]; then
       longCrit "$(logInfo)" "Init API response failed - Response:$resp Message:$msgResp"
       exit
   fi
}

#******************************************************************************#
#module       : waitForTunnelEstablishment
#Description  : check for latest handshake and establish control tunnel
#******************************************************************************#
waitForTunnelEstablishment()
{
    local resp=1
    while [ $resp -eq 1 ]
    do
        longInfo "$(logInfo)" "Waiting for Handshake..!!"
        wg | grep "latest handshake"
        resp=`echo $?`
        sleep 2
    done
}

#******************************************************************************#
#module       : wireguardTunnelEstablishment
#arg1         : EM url
#arg2         : device MAC
#Description  : Control tunnel establishment
#return       : tunnelIP, serverIP
#******************************************************************************#
wireguardTunnelEstablishment()
{
    mgmtUrl=$1
    deviceMac=$2
    local retryCount=0

    devicePublicKeyInfo=`cat /opt/inseego/etc/devicePublicKey`
    devicePrivateKeyInfo=`cat /opt/inseego/etc/devicePrivateKey`

    longDebug "$(logInfo)" "Wireguard API Param - DevicePublicKey : $devicePublicKeyInfo"

    while :
    do
        if [ "$retryCount" -le 3 ]; then
            apiResp=$(curlExec "WgKeyExchange" '{"device_public_key":"'$devicePublicKeyInfo'","device_public_data_key":"","device_token":"'$deviceMac'"}')
            serverPubkey=`echo $apiResp | jq -r .wg_public_key`
            tunnelIP=`echo $apiResp | jq -r .wg_private_ip`
            serverIP=`echo $apiResp | jq -r .server_ip`
            ctrlTunnelPort=`echo $apiResp | jq -r .control_tunnel_port`
            [[ -z "$apiResp" && -z "$serverPubkey" && -z "$tunnelIP" && -z "$serverIP" && -z "$ctrlTunnelPort" ]] || break
            sleep 5
            retryCount=`expr $retryCount + 1`
            [[ "$retryCount" -le 3 ]] && \
                longCrit "$(logInfo)" "WgKeyExchange API response empty:" \
                    "Retrying... count-$retryCount"
        else
            longCrit "$(logInfo)" "WgKeyExchange API response failed..."
            exit
        fi
    done
    if [ `echo $apiResp | jq -r .success` == "false" ];then
        return
    fi

    longDebug "$(logInfo)" "Keys - server_Pubkey: $serverPubkey" \
        "tunnel_ip : $tunnelIP server_ip: $serverIP "
    longInfo "$(logInfo)" "Control Tunnel Creation & Route entries..!!"

    WG_KEY="$(cat /opt/inseego/etc/devicePrivateKey)"

    uci -q del network.wg0
    uci -q set network.wg0=interface
    uci -q set network.wg0.proto='wireguard'
    uci -q set network.wg0.private_key="$WG_KEY"
    uci -q add_list network.wg0.addresses="$tunnelIP/24"
    uci -q set network.wg0.listen_port=$ctrlTunnelPort

    uci -q del network.controlTunnel
    uci -q set network.controlTunnel=wireguard_wg0
    uci -q set network.controlTunnel.public_key="$serverPubkey"
    uci -q set network.controlTunnel.route_allowed_ips=1
    uci -q add_list network.controlTunnel.allowed_ips="$serverIP"
    uci -q set network.controlTunnel.endpoint_host="$mgmtUrl"
    uci -q set network.controlTunnel.endpoint_port=$ctrlTunnelPort
    uci -q set network.controlTunnel.persistent_keepalive=25
    uci commit network

    echo "$tunnelIP $serverIP"
}

#******************************************************************************#
#module       : handShakeAPI
#Description  : ZTP handshake process
#Return       : handshake response
#******************************************************************************#
handShakeAPI()
{
    local retryCount=0

    longInfo "$(logInfo)" "Fetching of the Device details..!!"
    imei=`sed -n '1p' /opt/inseego/etc/device_details`
    model=`sed -n '2p' /opt/inseego/etc/device_details`
    serialID=`sed -n '3p' /opt/inseego/etc/device_details`
    platform=`uci get ztp.EM_details.platform`
    devicePublicKey=`cat /opt/inseego/etc/devicePublicKey`
    fw_version=`cat /etc/version`
    is_deleted=`cat /opt/inseego/etc/deleteStatus`
    supportedFeatureList=`cat /opt/inseego/etc/supportedFeatureList.json`

    longDebug "$(logInfo)" "HandshakeAPI param - device_mac:$deviceMac" \
        "device_serial:$serialID hostname:$dHostName device_model:$model" \
        "wg_pubkey:$devicePublicKey md5_sum:$md5sum device_imei:$imei" \
        "fw_version:$fw_version is_deleted:$is_deleted zHostname:$hostName"

    while :
    do
        if [ "$retryCount" -le 3 ]; then
	    apiResp=$(curlExec "connectDevice" '{"device_mac":"'$deviceMac'","device_serial":"'$serialID'","hostname":"'$dHostName'","device_model":"'$model'","wg_pubkey":"'$devicePublicKey'","device_public_data_key":"","md5_sum":"'$md5sum'","device_imei":"'$imei'","fw_version":"'$fw_version'","is_deleted":"'$is_deleted'","zHostname":"'$hostName'","is_headendcpe":"1","platform":"'$platform'","supported_templates":'$supportedFeatureList',"zabbix_template_version":"2"}')
            [[ -z "$apiResp" ]] || break
            sleep 5
            retryCount=`expr $retryCount + 1`
            [[ "$retryCount" -le 3 ]] && \
                longCrit "$(logInfo)" "connectDevice API response empty:" \
                    "Retrying... count-$retryCount"
        else
            longCrit "$(logInfo)" "connectDevice API response failed - Response:$resp"
            longInfo "$(logInfo)" "Removing ZTP Configurations"
            removeZTPfiles
            exit
        fi
    done
    echo "$apiResp" | jq -r .message
}

#******************************************************************************#
#module       : EMCheck
#Description  : checking EM reachability
#******************************************************************************#
EMCheck()
{
    resp=1
    while [ $resp -eq 1 ]
    do
        longInfo "$(logInfo)" "Waiting for EM reachability..!!"
        nc -z -v -w 5 $mgmtUrl $mgmtPort
        resp=`echo $?`
        sleep 1
    done
}

#******************************************************************************#
#module       : generateKey
#Description  : generating wireguard control tunnel key pair
#******************************************************************************#
generateKey()
{
    devicePrivateKey="$(wg genkey)"
    echo $devicePrivateKey > /opt/inseego/etc/devicePrivateKey
    devicePublicKey=`wg pubkey <<<"$devicePrivateKey"`
    echo $devicePublicKey > /opt/inseego/etc/devicePublicKey
}

#******************************************************************************#
#module       : generateDeviceDetails
#Description  : generating device details
#******************************************************************************#
generateDeviceDetails()
{
    longInfo "$(logInfo)" "Fetching IMEI of the Device..!!"
    imei="NA"
    echo >&2 $imei >> /opt/inseego/etc/device_details

    longInfo "$(logInfo)" "Fetching Model of the Device..!!"
    model='V2000'
    echo >&2 $model >> /opt/inseego/etc/device_details

    longInfo "$(logInfo)" "Fetching Serial ID of the Device..!!"
    serial_ID='FN0000000V2000'
    echo >&2 $serial_ID >> /opt/inseego/etc/device_details
}

#******************************************************************************#
#module       : stopCron
#Description  : removing ztp job from crontab.
#******************************************************************************#
stopCron() {
    longInfo "$(logInfo)" "Stop Cronjob activity"
    sed -i '/exe_ztp/d' /etc/crontabs/root
    /etc/init.d/cron restart
}

#ZTP Code Starts Here

lockDir="/tmp/ztp.lock"
if mkdir "$lockDir"
then    # directory did not exist, but was created successfully
    trap 'rm -rf "$lockDir"' 0    # remove directory when script finishes
else
  longEmerg "$(logInfo)" "ZTP already Running... Exiting..."
  exit
fi

#This arguments are recieved in case of bootstrap reset
factorystatus=$1
factoryoperid=$2

keyFile="/opt/inseego/etc/key"
certFile="/etc/ssl/cert.pem"
zabbixConf="/etc/zabbix_agentd.conf"
md5sumFile="/opt/inseego/etc/md5sumFile"
md5sum=`cat $md5sumFile | base64 --decode | md5sum | cut -d " " -f1`

mgmtProto=`uci -q get ztp.EM_details.proto`
if [ -z $mgmtProto ]; then
    longErr "$(logInfo)" "Edge Manager IP not Found..."
    exit
fi

mgmtUrl=`uci -q get ztp.EM_details.domain`
if [ -z $mgmtUrl ]; then
    longErr "$(logInfo)" "Edge Manager URL not Found..."
    exit
fi

mgmtPort=`uci -q get ztp.EM_details.port`
if [ -z $mgmtPort ]; then
    longErr "$(logInfo)" "Edge Manager Port not Found..."
    exit
fi

if [ ! -f "/opt/inseego/etc/devicePublicKey" ]; then
    longInfo "$(logInfo)" "Creating Private & Public Key"
    generateKey
fi

if [ ! -f "/opt/inseego/etc/device_details" ]; then
    longInfo "$(logInfo)" "Creating Device details"
    generateDeviceDetails
fi

longInfo "$(logInfo)" "Checking EM reachability from the device..!!"
EMCheck

longInfo "$(logInfo)" "Fetching Device MAC Address..!!"
deviceMac=`cat /sys/class/net/eth0/address`

longInfo "$(logInfo)" "Fetching Device Hostname..!!"
model=`sed -n '2p' /opt/inseego/etc/device_details`
dHostName=$model-$(echo $deviceMac |  cut -d: -f5- | tr -d ":" )
uci set system.@system[0].hostname=$dHostName
uci commit system
/etc/init.d/system restart

if [ ! -f /opt/inseego/etc/zabbix_hostname ]; then
    longInfo "$(logInfo)" "Fetching Zabbix Hostname..!!"
    hostName=$model-$(echo $((1000 + RANDOM % 9999)))-$(echo $deviceMac |  cut -d: -f5- | tr -d ":" )
    echo $hostName > /opt/inseego/etc/zabbix_hostname
fi

hostName=`cat /opt/inseego/etc/zabbix_hostname`

if [ ! -z $factorystatus ] && [ $factorystatus -eq 1 ]; then
    longInfo "$(logInfo)" "Updating Password for root"
    userName="root"
    password=`cat /opt/inseego/etc/password`
    password=$(decData $password)
    updatePasswd $password

    longInfo "$(logInfo)" "Generating the AccessKey..!!"
    genAccesskey
    longInfo "$(logInfo)" "Applying the Default Templates when factory reset"
    ztp_default_template_api $factoryoperid
    rm -f /operid
    exit
fi

longInfo "$(logInfo)" "Generating the AccessKey..!!"
genAccesskey

longInfo "$(logInfo)" "Initiate Handshake API"
hresp=$(handShakeAPI)
if [ ! "$hresp" == "Approved" ]; then
    if [ "$hresp" == "Online" ]; then
        longInfo "$(logInfo)" "Handshake API - Device is Online"
        stopCron
        exit
    fi
    if [ "$hresp" == "Offline" ]; then
        longInfo "$(logInfo)" "Handshake API - Device is Offline"
        stopCron
        exit
    fi
    if [ "$hresp" == "Deleted" ]; then
        longWarn "$(logInfo)" "Handshake API - Device is Deleted"
        echo "1" > /opt/inseego/etc/deleteStatus
        hresp=$(handShakeAPI)
        if [ "$hresp" == "SuccessfullyDeleted" ]; then
            longWarn "$(logInfo)" "Handshake API - Device is Deleted from EM:-Factory Resetting Device..."
            echo "0" > /isFactory
            echo "0" > /opt/inseego/etc/deleteStatus
            #To restore all the default configurations
            /sbin/firstboot && reboot
        fi
    fi
    if [ "$hresp" == "Duplicate" ]; then
        longWarn "$(logInfo)" "Handshake API - Device is Duplicated"
        echo "1" > /opt/inseego/etc/deleteStatus
        longInfo "$(logInfo)" "Sending delete status to EM..!!"
        hresp=$(handShakeAPI)
        if [ "$hresp" == "SuccessfullyDeleted" ]; then
            longWarn "$(logInfo)" "Handshake API - Device is Deleted. Onboarding again"
            echo "0" > /opt/inseego/etc/deleteStatus
            hresp=$(handShakeAPI)
        fi
    fi
    if [ "$hresp" == "Saved" ]; then
        longWarn "$(logInfo)" "Device Not Approved"
        grep "exe_ztp" /etc/crontabs/root
        rett=`echo $?`
        if [ $rett -ne 0 ]; then
            echo "*/1 * * * * bash /opt/inseego/bin/exe_ztp.sh" >> /etc/crontabs/root
            /etc/init.d/cron restart
        fi
    fi

    longInfo "$(logInfo)" "Saved State"
    exit
fi

longInfo "$(logInfo)" "Initiating Tunnel Establishment..!!"
data=$(wireguardTunnelEstablishment $mgmtUrl $deviceMac)
if [ -z "$data" ]; then
    longInfo "$(logInfo)" "wireguardTunnelEstablishment empty: "
    exit
fi
tunnelIP=$(echo $data | cut -d ' ' -f1)
serverIP=$(echo $data | cut -d ' ' -f2)

longInfo "$(logInfo)" "Sending Device Info to Edge Manager..!!"
sendDeviceInfo

longInfo "$(logInfo)" "Configure Zabbix Agent with new config..!!"
configureZabbixAgent $serverIP $hostName

#Stop Cron Job
stopCron

/etc/init.d/network reload
ifdown wg0
ifup wg0
longInfo "$(logInfo)" "Wait until Tunnel Establish Successfully..!!"
waitForTunnelEstablishment

longInfo "$(logInfo)" "Generating Initial wireguard Key for Data Tunnel..!!"
/opt/inseego/bin/dataTunnelKey.sh "Bearer ${accessKey}"

longInfo "$(logInfo)" "Configuring NTP client"
uci -q delete system.ntp.server
uci set system.ntp.server="$serverIP 0.openwrt.pool.ntp.org"
uci commit system
/etc/init.d/sysntpd restart

longNotice "$(logInfo)" "Applying the Default Templates when onboarding"
ztp_default_template_api
