#!/bin/bash

. /lib/functions.sh "ADMIN"
. /opt/inseego/bin/sdedge_common_params.sh

#Fetch the Caller Function name and Line number
logInfo() { echo "("${FUNCNAME[1]}":"${BASH_LINENO}")"; }

#******************************************************************************#
#module       : curlExec
#arg1         : API need to be called
#arg2         : Payload for API
#Description  : Used to call the API
#Return       : API response
#******************************************************************************#
curlExec()
{
    local apiName=$1
    local datarawParam=$2
    local apiResp=""
    local curlResp=""

    apiResp=$(curl --cacert $certFile $allowK --location \
        --connect-timeout $timeout -m $maxtimeout --request POST \
        $mgmtUrl:$mgmtPort/api-device/device/$apiName \
        --header  'Authorization: '"$Key"'' --header \
        'Content-Type: application/json' --data-raw $datarawParam --header "X-Tenant-Name: $tenName")
    curlResp="$?"
    longDebug "$(logInfo)" "Curl Exit code for $apiName API - $curlResp"
    longDebug "$(logInfo)" "$apiName Response: $apiResp"
    echo $apiResp
}

genAccesskey()
{
    #Generating the access Token
    longInfo "$(logInfo)" "Generating Access Tokens...!!"
    /opt/inseego/bin/access.sh
    accessKey=`jq -r .access $keyFile`
    if [ -z "$accessKey" -o "$accessKey" = "null" ]
    then
        longErr "$(logInfo)" "AccessKey is empty - $accessKey Exiting..."
        exit
    fi
    Key="Bearer ${accessKey}"
}
#******************************************************************************#
#module       : Firmware upgarde API
#arg1         : Operation ID of process received from EM
#arg2         : Severity of message send to EM
#arg3         : Remark message send to EM(must be single or with underscore)
#Description  : Used to send execution status of Firmware upgrade to EM
#******************************************************************************#
fw_api()
{
    operid=$1
    severity=$2
    remarks=$3
    local retryCount=0

    while :
    do
        if [ "$retryCount" -le 3 ]; then
            apiResp=$(curlExec "firmwareUpgradeStatus" \
                '{"operation_id":"'$operid'","severity":"'$severity'","remarks":"'$remarks'","apiKey":"'$apiKey'"}')
            [[ -z "$apiResp" ]] || break
            sleep 5
            retryCount=`expr $retryCount + 1`
            [[ $retryCount -le 3 ]] && \
                longCrit "$(logInfo)" "Firmware upgrade response empty: "\
                    "Retrying... count-$retryCount"
        else
            longCrit "$(logInfo)" "Firmware upgrade Response Failed... Exiting..."
            exit
        fi
    done
    longInfo "$(logInfo)" "${operid} Message Sent Successful"
}

#******************************************************************************#
#module       : Upgrading FOTA
#arg1         : md5sum of tar file
#arg2         : Operation ID of process received from EM
#Description  : To upgrade the image
#******************************************************************************#
upgrade()
{
    if [ "$#" -ne 2 ]; then
        longErr "$(logInfo)" "Insufficient Argument from playbook... Exiting..."
        fw_api "${operid}" "Error" "Insufficient_Argument_from_playbook"
        exit
    fi
    md5sum=$1
    operid=$2
    longNotice "$(logInfo)" "Firmware Upgrade Execution Started" \
        "md5sum of tar file=$md5sum operation id=$operid"

    #Download Firmware File
    rm /tmp/firmware.tar.gz > /dev/null 2>&1
    apiResp=$(curl --cacert $certFile $allowK --connect-timeout $timeout -m $maxtimeout --retry $retry \
        --retry-delay $retryDelay --header 'Authorization: '"$Key"'' --output /tmp/firmware.tar.gz \
        $mgmtUrl:$mgmtPort/api-device/device/firmwareDownload?operation_id=$operid)
    curlResp=$?
    longInfo "$(logInfo)" "Curl Exit code for Firmware download - "$curlResp""
    if [ "$curlResp" -ne 0 ] && [ "$curlResp" -ne 28 ]; then
        longEmerg "$(logInfo)" "Firmware Download Failed... Exiting"
        fw_api "${operid}" "Error" "Firmware_Download_Failed..."
        exit
    fi

    #Checking for Downloaded File
    if [ ! -f /tmp/firmware.tar.gz ]; then
        longCrit "$(logInfo)" "Image not Exists... Exiting"
        fw_api "${operid}" "Error" "Image_not_Exists"
        exit
    fi

    dwnld_md5sum=`md5sum /tmp/firmware.tar.gz |awk '{print$1}'`
    maxtimeout=300
    retry=0
    while [ $md5sum != $dwnld_md5sum ] && [[ $timeout -le 900 ]] && [[ $retry -le 5 ]];
    do
        genAccesskey
        longCrit "$(logInfo)" "MD5SUM Mismatched-Retrying with timeout of $timeout seconds"
        apiResp=$(curl --cacert $certFile $allowK --connect-timeout $timeout -m $maxtimeout  \
           --header 'Authorization: '"$Key"'' --output /tmp/firmware.tar.gz \
            $mgmtUrl:$mgmtPort/api-device/device/firmwareDownload?operation_id=$operid)
        curlResp=$?
        longInfo "$(logInfo)" "Curl Exit code for Firmware download - "$curlResp""
        retry=$((retry + 1 ))
        dwnld_md5sum=`md5sum /tmp/firmware.tar.gz |awk '{print$1}'`
        [[ $md5sum == $dwnld_md5sum ]] && { longInfo "$(logInfo)" "MD5SUM matched - Firmware downloaded successfully" && break; }
        [[ $md5sum != $dwnld_md5sum ]] && { [[ $curlResp == 28 ]] && timeout=$((timeout * 2)); }
        longInfo "$(logInfo)" "Downloading Firmware with retry count - "$retry""
    done

    #Extracting the File
    rm /tmp/*.img > /dev/null 2>&1
    longNotice "$(logInfo)" "Extracting the Firmware Image"
    tar -xzvf /tmp/firmware.tar.gz -C /tmp/ > /dev/null 2>&1

    #Checking for Extracted File
    if [ ! -f /tmp/vchimay*.img ]; then
        longErr "$(logInfo)" "Image not Exists in tar file"
        fw_api "${operid}" "Error" "Image_not_Exists_in_tar_file"
        exit
    fi

    #Upgrading Image in Device
    longInfo "$(logInfo)" "Firmware upgrade in Progress"
    fw_api "${operid}" "Info" "Firmware_Upgrade_in_Progress"
    sysupgrade /tmp/vchimay*.img
    if [ "$?" -eq 1 ]; then
        longEmerg "$(logInfo)" "Firmware upgrade Failed..."
        fw_api "${operid}" "Error" "Firmware_Upgrade_Failed..."
    fi
}

genAccesskey

upgrade $@
