#!/bin/bash

# Script for updating data tunnel parameters tx_bytes, rx_bytes for 
# wan and wireguard interfaces & added delete logic to remove unused
# interface sections.

. /lib/functions.sh
uci_cmd="uci -q"
# Collecting wan wireguard interfaces from wg show output and network config file.
wg_ifaces="$(wg | grep interface | awk '{print $2}' | tr '\n' ' ')"
wan_ifaces=$($uci_cmd get network.wan_entry.usemember)
intfList="$wg_ifaces$wan_ifaces"
if [[ ! -z $wan_ifaces ]]; then
	$uci_cmd set ifaces_statistics.config.intf_list="$intfList"
else
	#To make the value of wan data usage get increased(not making as zero) even after bootstrap resetting the device.
	#In the presistent_data_usage script wan data usage will be added for all the wan entries.
	$uci_cmd set ifaces_statistics.config.intf_list="wan wg0"
	intfList=`$uci_cmd get ifaces_statistics.config.intf_list`
fi
params="rx_bytes tx_bytes"
old_count=0
count=0
total_uci_data=0
last_sample_data=0

#*************
#@func: config_cleaner
#@desc: will delete unused interfaces section from config file which are deleted by user and are unused now.
#*************
config_cleaner()
{
	old_ifaces=$(uci -q get ifaces_statistics.config.intf_list)
	for intf in $old_ifaces; do
		#To delete all other interfaces except control tunnel and wan
		if ! [[ "$intfList" =~ "$intf" ]] && [[ "${intf:0:3}" != "wan" ]] && [[ "$intf" != "wg0" ]]; then
			echo "did not found, $intf can be deleted"
			uci delete ifaces_statistics.$intf
		fi
	done
	uci commit ifaces_statistics
}

#*********
# @func: update_statistics
# @desc: will update interface statistics params like rx_bytes,tx_bytes,rx_packets,tx_packets
# @desc: in ifaces_statistics config file.
#********
update_statistics()
{
	for intf in $intfList; do
		# check whether interface section is present, if not then create with some default details.
		status=$($uci_cmd show ifaces_statistics.$intf)
		if [ $? != 0 ];	then
			$uci_cmd set ifaces_statistics.$intf=interface
			interface=`$uci_cmd get network.$intf.ifname`
			$uci_cmd set ifaces_statistics.$intf.ifname=$interface
			for param_type in $params; do
				$uci_cmd set ifaces_statistics.$intf.$param_type='0'
				$uci_cmd set ifaces_statistics.$intf.last_sample_$param_type='0'
			done
		fi
		for param_type in $params; do
			# differentiate wireguard and wan interfaces and collecting param details for them.
			if [ "${intf:0:2}" == "wg" ]; then
				proc_data=`cat /sys/class/net/$intf/statistics/$param_type`
				total_uci_data=$($uci_cmd get ifaces_statistics."$intf".$param_type)
				last_sample_data=$($uci_cmd get ifaces_statistics."$intf".last_sample_$param_type)
			else
				proc_data=`cat /sys/class/net/$($uci_cmd get network.$intf.ifname)/statistics/$param_type`
				total_uci_data=$($uci_cmd get ifaces_statistics."$intf".$param_type)
				last_sample_data=$($uci_cmd get ifaces_statistics."$intf".last_sample_$param_type)
			fi

			# if details are present then update.
			uptime="$(uptime)"
			if [ "$proc_data" != "" -a "$last_sample_data" != "" -a "$total_uci_data" != "" ]; then
				if [[ $proc_data -ge $total_uci_data ]]; then
					#Exceptional case: After Device Reboot, when proc_data is greater than total_uci_data
					#In reboot or bootstrap reset case, uptime is less than or equal to 1 minute and the 4th variabe is "min"
					if [[ `echo $uptime | awk '{print $3}' | cut -d ':' -f1` -le "1" ]] && [[ `echo $uptime | awk '{print $4}' | cut -d "," -f1` == "min" ]]; then
						diff=`expr $proc_data`
					else
						diff=`expr $proc_data - $last_sample_data`
					fi
				#After Device Reboot, when proc_data is less than total_uci_data
				else
					if [[ $total_uci_data != $last_sample_data ]] && [[ $proc_data -ge $last_sample_data ]]; then
						diff=`expr $proc_data - $last_sample_data`
					else
						diff=`expr $proc_data`
					fi
				fi
				$uci_cmd set ifaces_statistics."$intf".$param_type=`expr $total_uci_data + $diff`
				$uci_cmd set ifaces_statistics."$intf".last_sample_$param_type="$proc_data"
				total_uci_data=0
				last_sample_data=0
				proc_data=""
			fi
		done
		count=$((count+1))
	done

	# comparing count of new and old interfaces, if old count has more interfaces, then find unused and delete.
	old_count="$(uci -q get ifaces_statistics.config.count)"
	if [ $old_count -gt $count ]; then
		config_cleaner
	fi
	$uci_cmd set ifaces_statistics.config.count="$count"
	$uci_cmd commit ifaces_statistics
	chmod 777 /etc/config/ifaces_statistics
}

stop ()
{
	(crontab -l 2>/dev/null | grep -v ifaces_update.sh) | crontab -
	/etc/init.d/cron restart
}

start ()
{
	stop
	echo "Initializing on $date -R)."
	(crontab -l 2>/dev/null; echo "*/1 * * * * /opt/inseego/bin/ifaces_update.sh") | crontab -
	/etc/init.d/cron restart
	update_statistics
}

case $1 in
	start) start;;
	stop) stop;;
	*) update_statistics ;;
esac
